_G=GLOBAL
require=_G.require
local cooking = require "cooking"
local cookpot = cooking and cooking.recipes.cookpot --or {}
local ingredients = cooking.ingredients --or {}

TUNING.SNAKEOIL_SMELL_MULT = 0.6
TUNING.POISON_SMELL_MULT = 0.97
TUNING.ROT_SMELL_MULT = 1.03 --also poop
TUNING.MONSTERMEAT_SMELL_MULT = 1.01

TUNING.SPOIL_SMELL_MULT_MIN = 0.3
TUNING.SPOIL_SMELL_MULT_MAX = 2.5

TUNING.SNAKEOIL_ADD_BOAT_HEALTH = 75
TUNING.SNAKEOIL_ADD_BOAT_PERISH = TUNING.TOTAL_DAY_TIME --we want to get limit in 75 per day (as raft).

local function UpdateContainer(inst) --inst = chest
	local items
	if inst.components.container then
		items = inst.components.container:FindItems(function(v) return v.snake_smell end)
	elseif inst.components.inventory then
		local c = inst.components.inventory
		items = c:FindItems(function(v) return v.snake_smell and v.components.inventoryitem.owner==inst and v~=c.activeitem end)
	end
	if not items then
		return
	end
	local mult = 1
	for i,v in ipairs(items) do
		for j=1,v.components.stackable.stacksize do
			mult = mult * v.snake_smell
		end
	end
	if mult < TUNING.SPOIL_SMELL_MULT_MIN then
		mult = TUNING.SPOIL_SMELL_MULT_MIN
	elseif mult > TUNING.SPOIL_SMELL_MULT_MAX then
		mult = TUNING.SPOIL_SMELL_MULT_MAX
	end
	inst.snakeoil_smell = mult ~= 1 and mult or nil
end

local function DropFridge(snakeoil)
	local owner = snakeoil.fridge
	if owner and owner.snakeoil_smell then --and owner:IsValid() then
		UpdateContainer(owner)
	end
	snakeoil.fridge = nil
end 


local function PutInInventoryFn(inst)
	local owner = inst.components.inventoryitem.owner
	if owner then
		inst:DoTaskInTime(0,function()
		UpdateContainer(owner)
		end)
		inst.fridge = owner
	end
end

local function MakeItemSmell(inst,power)
	inst:AddTag("icebox_valid")
	
	if not inst.components.stackable then
		inst:AddComponent("stackable")
	end
	inst:ListenForEvent("stacksizechange",PutInInventoryFn)
	
	inst.components.inventoryitem:SetOnDroppedFn(DropFridge)
	inst.components.inventoryitem:SetOnActiveItemFn(DropFridge)
	inst.components.inventoryitem:SetOnPickupFn(DropFridge)
	inst.components.inventoryitem:SetOnPutInInventoryFn(PutInInventoryFn)
	
	local old_onremove = inst.OnRemoveEntity
	inst.OnRemoveEntity = function(inst,...)
		DropFridge(inst)
		if old_onremove then
			old_onremove(inst,...)
		end
	end
	
	inst.snake_smell = power
end

local function AddSmell(prefab,power)
	AddPrefabPostInit(prefab,function(inst)
		MakeItemSmell(inst,power)
	end)
end

AddSmell("venomgland",TUNING.POISON_SMELL_MULT)
AddSmell("antivenom",TUNING.POISON_SMELL_MULT)
AddSmell("spoiled_food",TUNING.ROT_SMELL_MULT)
AddSmell("poop",TUNING.ROT_SMELL_MULT)
AddSmell("monstermeat",TUNING.MONSTERMEAT_SMELL_MULT)

AddPrefabPostInit("snakeoil",function(inst)
	if not inst.components.poisonhealer then
		inst:AddComponent("poisonhealer")
	end
	inst.components.poisonhealer.enabled = true
	
	if not inst.components.fuel then
		inst:AddComponent("fuel")
	end
	inst.components.fuel.fuelvalue = TUNING.LARGE_FUEL
	
	inst:AddComponent("repairer")
	inst.components.repairer.repairmaterial = "boat"
	inst.components.repairer.healthrepairvalue = TUNING.SNAKEOIL_ADD_BOAT_HEALTH * 2 --del: TUNING.BOAT_REPAIR_KIT_HEALING * 3
	
	MakeItemSmell(inst,TUNING.SNAKEOIL_SMELL_MULT)
end)


--Improve the boat on repair by oil
local repairable = require "components/repairable"
local old_Repair = repairable.Repair
function repairable:Repair(doer, repair_item,...)
	--print("REPAIR")
	if self.inst:HasTag("boat") and self.inst.components.boathealth and repair_item.prefab == "snakeoil" then
		local bh = self.inst.components.boathealth
		--print("BOAT",tostring(bh.oil_base_health),tostring(bh.inst),self)
		bh.grease_times = bh.grease_times or 0
		if bh.grease_times < 5 then
			--print("MORE TIMES",bh.grease_times)
			bh.grease_times = bh.grease_times + 1
			bh:UpgradeOil()
		end
	end
	return old_Repair(self,doer, repair_item,...)
end

local bh = require "components/boathealth"
local old_SetHealth = bh.SetHealth
function bh:SetHealth(hp, perishtime,...)
	--print("SetHealth",hp,tostring(perishtime),self.inst,self)
	if perishtime and not self.base_perishtime then
		--print("Save base halth!")
		self.base_perishtime = perishtime
		self.oil_base_health = hp
		--print(tostring(self.oil_base_health))
		if self.grease_times then
			self:UpgradeOil()
		end
	end
	return old_SetHealth(self,hp, perishtime,...)
end

local old_bh_OnSave = bh.OnSave
function bh:OnSave(...)
	local res = old_bh_OnSave(self,...)
	if res and self.oil_base_health then
		--res.oil_base_health = self.oil_base_health
		--res.base_perishtime = self.base_perishtime
		res.grease_times = self.grease_times
	end
	return res
end

function bh:SaveBaseHealth()
	
end

local old_bh_OnLoad = bh.OnLoad
function bh:OnLoad(data,...)
	if data then
		--self.oil_base_health = data.oil_base_health or self.oil_base_health
		--self.base_perishtime = data.base_perishtime or self.base_perishtime
		self.grease_times = data.grease_times or self.grease_times
	end
	old_bh_OnLoad(self,data,...)
	if self.oil_base_health and self.grease_times then
		self:UpgradeOil()
	end
end

function bh:UpgradeOil()
	--print("UPGRADE",tostring(self.oil_base_health),tostring(self.grease_times),tostring(self.inst),self)
	if not self.oil_base_health then
		--print("avoiding crash")
		return
	end
	self.maxhealth = self.oil_base_health + self.grease_times * TUNING.SNAKEOIL_ADD_BOAT_HEALTH
	self.inst.perishtime = self.base_perishtime + self.grease_times * TUNING.SNAKEOIL_ADD_BOAT_PERISH
	self:SetHealth(self.currenthealth, self.inst.perishtime)
	self.rate = self.maxhealth / self.inst.perishtime
	--self:DoDelta(TUNING.SNAKEOIL_ADD_BOAT_HEALTH*0.5, "repair", repair_item, true)
	if self.inst.waveboost then
		self.inst.waveboost = math.max(self.inst.waveboost,TUNING.WAVEBOOST+10)
	end
end


---Calc fn
local function CalculateSmellMult(item,owner)
	if not item or item:HasTag("frozen") then
		return
	end
	local owner = item and item.components.inventoryitem and item.components.inventoryitem.owner
	if owner and owner.snakeoil_smell then
		return owner.snakeoil_smell
	end
end


--Tuning perishable component
local perishable = require "components/perishable"
local old_LongUpdate = perishable.LongUpdate
function perishable:LongUpdate(dt, ...)
	local mult = CalculateSmellMult(self.inst)
	if mult then
		local save = TUNING.PERISH_GLOBAL_MULT
		TUNING.PERISH_GLOBAL_MULT = mult
		old_LongUpdate(self,dt,...)
		TUNING.PERISH_GLOBAL_MULT = save
		--print("LongUpdate: ",inst.prefab,mult)
		return
	end
	return old_LongUpdate(self,dt,...)
end

local function myPerishUpdate(inst, dt, ...)
	local mult = CalculateSmellMult(inst)
	if mult then
		local save = TUNING.PERISH_GLOBAL_MULT
		TUNING.PERISH_GLOBAL_MULT = mult
		inst.snakeoil_save_updatetask_fn(inst, dt, ...)
		TUNING.PERISH_GLOBAL_MULT = save
		--print("Update: ",inst.prefab,mult)
		return
	end
	return inst.snakeoil_save_updatetask_fn(inst, dt, ...)
end

local old_StartPerishing = perishable.StartPerishing
function perishable:StartPerishing(...)
	old_StartPerishing(self,...)
	if not self.updatetask then
		return
	end
	self.inst.snakeoil_save_updatetask_fn = self.updatetask.fn
	self.updatetask.fn = myPerishUpdate
end


--Set fire on heal
--[[
local poisonable = require "components/poisonable"
local old_Cure = poisonable.Cure
function poisonable:Cure(curer, ...)
	if curer and curer.snake_smell and curer.snake_smell < 0.8 and self.inst.components.burnable then
		self.inst.components.burnable:Ignite()
	else
		--print("curer",curer)
		--print("curer.snake_smell",curer.snake_smell)
		--print("curer.snake_smell < 0.8",curer.snake_smell < 0.8)
		--print("self.inst.components.burnable",self.inst.components.burnable)
	end
	return old_Cure(self,curer,...)
end
--]]

--TUNING.PERISH_FRIDGE_MULT = 0.3

--Fixing oil ingredients

local function AddIngredientValues(names, tags, cancook, candry)
	for _,name in pairs(names) do
		if not ingredients[name] then --Do not replace existing values.
			ingredients[name] = { tags= {}}
		end

		if cancook and not ingredients[name.."_cooked"] then --Do not replace existing values.
			ingredients[name.."_cooked"] = {tags={precook = 1}}
		end

		if candry and not ingredients[name.."_dried"] then --Do not replace existing values.
			ingredients[name.."_dried"] = {tags={dried = 1}}
		end

		for tagname,tagval in pairs(tags) do
			ingredients[name].tags[tagname] = tagval
			--print(name,tagname,tagval,ingtable[name].tags[tagname])

			if cancook then
				ingredients[name.."_cooked"].tags[tagname] = tagval
			end
			if candry then
				ingredients[name.."_dried"].tags[tagname] = tagval
			end
		end
	end
end

AddIngredientValues({"snakeoil"}, {fat=1})
AddIngredientValues({"lobster_dead"}, {fish=2}, true)
AddIngredientValues({"jellyfish_dead"}, {fish=1,monster=1})

--Fixing recipes

if cookpot.lobsterdinner then
	local old_test = cookpot.lobsterdinner.test
	cookpot.lobsterdinner.test = function(cooker, names, tags)
		return ((names.lobster or names.lobster_dead or names.lobster_dead_cooked) and tags.fat and not tags.meat and not tags.frozen)
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

--Some small fixes and improvements:

if cookpot.lobsterbisque then
	local old_test = cookpot.lobsterbisque.test
	cookpot.lobsterbisque.test = function(cooker, names, tags)
		return ((names.lobster or names.lobster_dead or names.lobster_dead_cooked) and tags.frozen) --stronger condition
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.bananapop then
	local old_test = cookpot.bananapop.test
	cookpot.bananapop.test = function(cooker, names, tags)
		return ((names.cave_banana or names.cave_banana_cooked) and tags.frozen and tags.inedible and not tags.meat and not tags.fish)
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.jellyopop then
	local old_test = cookpot.jellyopop.test
	cookpot.jellyopop.test = function(cooker, names, tags)
		return ((names.jellyfish or names.jellyfish_dead or names.jellyfish_cooked or names.jellyjerky) and tags.frozen and tags.inedible)
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.bisque then
	local old_test = cookpot.bisque.test
	cookpot.bisque.test = function(cooker, names, tags)
		return ((names.limpets or 0)+(names.limpets_cooked or 0) == 3 and tags.frozen) --stronger condition
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.californiaroll then
	local old_test = cookpot.californiaroll.test
	cookpot.californiaroll.test = function(cooker, names, tags)
		return ((names.seaweed or 0)+(names.seaweed_cooked or 0)+(names.seaweed_dried or 0)>=2 and (tags.fish and tags.fish >= 1))
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.watermelonicle then
	local old_test = cookpot.watermelonicle.test
	cookpot.watermelonicle.test = function(cooker, names, tags)
		return ((names.watermelon or names.watermelon_cooked) and tags.frozen and names.twigs and not tags.meat and not tags.veggie and not tags.egg)
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.waffles then
	local old_test = cookpot.waffles.test
	cookpot.waffles.test = function(cooker, names, tags)
		return (tags.fat and (names.berries or names.berries_cooked or names.pomegranate or names.pomegranate_cooked) and tags.egg)
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.turkeydinner then
	local old_test = cookpot.turkeydinner.test
	cookpot.turkeydinner.test = function(cooker, names, tags)
		return ((names.drumstick or 0)+(names.drumstick_cooked or 0) > 1 and tags.meat and tags.meat > 1 and (tags.veggie or tags.fruit)) --stronger condition
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.mandrakesoup then
	local old_test = cookpot.mandrakesoup.test
	cookpot.mandrakesoup.test = function(cooker, names, tags)
		return (names.mandrake or names.mandrake_cooked) --stronger condition
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.honeyham then
	local old_test = cookpot.honeyham.test
	cookpot.honeyham.test = function(cooker, names, tags)
		return (tags.sweetener and tags.meat and tags.meat > 1.5 and not tags.inedible) --stronger condition
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.honeynuggets then
	local old_test = cookpot.honeynuggets.test
	cookpot.honeynuggets.test = function(cooker, names, tags)
		return (tags.sweetener and tags.meat and tags.meat <= 1.5 and not tags.inedible) --stronger condition
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end

if cookpot.trailmix then
	local old_test = cookpot.trailmix.test
	cookpot.trailmix.test = function(cooker, names, tags)
		return ((names.coconut or names.coconut_cooked or (tags.seed and tags.seed >= 1)) and (names.berries or names.berries_cooked or names.pomegranate or names.pomegranate_cooked or names.cave_banana or names.cave_banana_cooked) and tags.fruit and tags.fruit >= 1 - (names.coconut or 0) - (names.coconut_cooked or 0) and not tags.meat and not tags.veggie and not tags.egg and not tags.dairy)
			or old_test(cooker, names, tags) --Compatibility with any future Klei fixes and updates.
	end
end


























